@@ -1,53 +1,65 @@
 # -*- coding: utf-8 -*-
 from AccessControl import getSecurityManager
+from Acquisition import aq_inner
+from plone.app.users.browser.account import AccountPanelSchemaAdapter
+from plone.app.users.browser.interfaces import ILoginNameGenerator
+from plone.app.users.browser.interfaces import IUserIdGenerator
+from plone.app.users.browser.schemaeditor import getFromBaseSchema
+from plone.app.users.schema import IAddUserSchema
+from plone.app.users.schema import ICombinedRegisterSchema
+from plone.app.users.schema import IRegisterSchema
+from plone.app.users.utils import notifyWidgetActionExecutionError
+from plone.app.users.utils import uuid_userid_generator
+from plone.autoform.form import AutoExtensibleForm
+from plone.protect import CheckAuthenticator
+from plone.registry.interfaces import IRegistry
 from Products.CMFCore.interfaces import ISiteRoot
 from Products.CMFCore.permissions import ManagePortal
 from Products.CMFCore.utils import getToolByName
 from Products.CMFPlone import PloneMessageFactory as _
+from Products.CMFPlone.interfaces import IPloneSiteRoot
 from Products.CMFPlone.interfaces import ISecuritySchema
+from Products.CMFPlone.interfaces import IUserGroupsSettingsSchema
+from Products.CMFPlone.utils import get_portal
 from Products.CMFPlone.utils import normalizeString
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 from Products.statusmessages.interfaces import IStatusMessage
-from ZODB.POSException import ConflictError
-from plone.autoform.form import AutoExtensibleForm
-from plone.autoform.interfaces import OMITTED_KEY
-from plone.protect import CheckAuthenticator
-from plone.registry.interfaces import IRegistry
 from z3c.form import button
 from z3c.form import field
 from z3c.form import form
 from z3c.form.browser.checkbox import CheckBoxFieldWidget
 from z3c.form.interfaces import DISPLAY_MODE
 from zExceptions import Forbidden
-from zope.component import (
-    getUtility,
-    queryUtility,
-    getAdapter,
-    getMultiAdapter)
-from zope.interface import Interface
+from ZODB.POSException import ConflictError
+from zope.component import getAdapter
+from zope.component import getMultiAdapter
+from zope.component import getUtility
+from zope.component import provideAdapter
+from zope.component import queryUtility
 from zope.schema import getFieldNames
+
 import logging
+import six
 
-from ..schema import (
-    IRegisterSchema,
-    IRegisterSchemaProvider,
-    IAddUserSchema,
-    ICombinedRegisterSchema,
-    IUserDataSchema)
-from ..utils import (
-    notifyWidgetActionExecutionError,
-    uuid_userid_generator,
-)
-
-from .account import AccountPanelSchemaAdapter
-
-from plone.app.users.browser.interfaces import ILoginNameGenerator
-from plone.app.users.browser.interfaces import IUserIdGenerator
 
 # Number of retries for creating a user id like bob-jones-42:
 RENAME_AFTER_CREATION_ATTEMPTS = 100
 
 
+def getRegisterSchema():
+    portal = get_portal()
+    schema = getattr(portal, '_v_register_schema', None)
+    if schema is None:
+        portal._v_register_schema = schema = getFromBaseSchema(
+            ICombinedRegisterSchema,
+            form_name=u'On Registration'
+        )
+        # as schema is a generated supermodel,
+        # needed adapters can only be registered at run time
+        provideAdapter(AccountPanelSchemaAdapter, (IPloneSiteRoot,), schema)
+    return schema
+
+
 class BaseRegistrationForm(AutoExtensibleForm, form.Form):
     """Form to be used as base for Register and Add User forms."""
     label = u""
@@ -55,14 +67,13 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
     formErrorsMessage = _('There were errors.')
     ignoreContext = True
     enableCSRFProtection = True
-    schema = ICombinedRegisterSchema
 
     # this attribute indicates if user was successfully registered
     _finishedRegister = False
 
-    def __init__(self, *args, **kwargs):
-        super(BaseRegistrationForm, self).__init__(*args, **kwargs)
-        self.schema = getUtility(IRegisterSchemaProvider).getSchema()
+    @property
+    def schema(self):
+        return getRegisterSchema()
 
     def _get_security_settings(self):
         """Return security settings from the registry."""
@@ -82,23 +93,6 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
         settings = self._get_security_settings()
         use_email_as_login = settings.use_email_as_login
 
-        # Filter schema for registration
-        omitted = []
-        default_fields = IUserDataSchema.names() + IRegisterSchema.names()
-        for name in self.schema:
-            # we always preserve default fields
-            if name in default_fields:
-                omit = False
-            else:
-                forms_selection = getattr(
-                    self.schema[name], 'forms_selection', [])
-                if u'On Registration' in forms_selection:
-                    omit = False
-                else:
-                    omit = True
-            omitted.append((Interface, name, omit))
-        self.schema.setTaggedValue(OMITTED_KEY, omitted)
-
         # Finally, let autoform process the schema and any FormExtenders do
         # their thing
         super(BaseRegistrationForm, self).updateFields()
@@ -304,7 +298,7 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
 
         # passwords should match
         if 'password' in form_field_names:
-            assert('password_ctl' in form_field_names)
+            assert 'password_ctl' in form_field_names
             # Skip this check if password fields already have an error
             if not ('password' in error_keys or 'password_ctl' in error_keys):
                 password = data.get('password')
@@ -407,8 +401,6 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
 
         self.handle_join_success(data)
 
-        self._finishedRegister = True
-
         # XXX Return somewhere else, depending on what
         # handle_join_success returns?
         came_from = self.request.form.get('came_from')
@@ -427,13 +419,14 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
         # user_id and login_name should be in the data, but let's be safe.
         user_id = data.get('user_id', data.get('username'))
         login_name = data.get('login_name', data.get('username'))
-        # I have seen a unicode user id.  I cannot reproduce it, but
-        # let's make them strings, otherwise you run into trouble with
-        # plone.session when trying to login.
-        if isinstance(user_id, unicode):
-            user_id = user_id.encode('utf8')
-        if isinstance(login_name, unicode):
-            login_name = login_name.encode('utf8')
+        if six.PY2:
+            # I have seen a unicode user id.  I cannot reproduce it, but
+            # let's make them strings, otherwise you run into trouble with
+            # plone.session when trying to login.
+            if isinstance(user_id, six.text_type):
+                user_id = user_id.encode('utf8')
+            if isinstance(login_name, six.text_type):
+                login_name = login_name.encode('utf8')
 
         # Set the username for good measure, as some code may expect
         # it to exist and contain the user id.
@@ -447,14 +440,15 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
         self.request.form['form.username'] = login_name
 
         password = data.get('password') or registration.generatePassword()
-        if isinstance(password, unicode):
+        if six.PY2 and isinstance(password, six.text_type):
             password = password.encode('utf8')
 
         try:
             registration.addMember(user_id, password, REQUEST=self.request)
-        except (AttributeError, ValueError), err:
+        except (AttributeError, ValueError) as err:
             logging.exception(err)
             IStatusMessage(self.request).addStatusMessage(err, type="error")
+            self._finishedRegister = False
             return
 
         if user_id != login_name:
@@ -467,6 +461,7 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
         self.applyProperties(user_id, data)
 
         settings = self._get_security_settings()
+        self._finishedRegister = True
         if data.get('mail_me') or (not settings.enable_user_pwd_choice and
                                    not data.get('password')):
             # We want to validate the email address (users cannot
@@ -484,7 +479,8 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
             except ConflictError:
                 # Let Zope handle this exception.
                 raise
-            except Exception:
+            except Exception as err:
+                logging.exception(err)
                 ctrlOverview = getMultiAdapter((portal, self.request),
                                                name='overview-controlpanel')
                 mail_settings_correct = not ctrlOverview.mailhost_warning()
@@ -495,6 +491,7 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
                     # Remove the account:
                     self.context.acl_users.userFolderDelUsers(
                         [user_id], REQUEST=self.request)
+                    self._finishedRegister = False
                     IStatusMessage(self.request).addStatusMessage(
                         _(u'status_fatal_password_mail',
                           default=u"Failed to create your account: we were "
@@ -502,7 +499,6 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
                           "to your email address: ${address}",
                           mapping={u'address': data.get('email', '')}),
                         type='error')
-                    return
                 else:
                     # This should only happen when an admin registers
                     # a user.  The admin should have seen a warning
@@ -514,9 +510,6 @@ class BaseRegistrationForm(AutoExtensibleForm, form.Form):
                           "password to this email address: ${address}",
                           mapping={u'address': data.get('email', '')}),
                         type='warning')
-                    return
-
-        return
 
     def applyProperties(self, userid, data):
         portal = getToolByName(self.context, 'portal_url').getPortalObject()
@@ -646,15 +639,15 @@ class AddUserForm(BaseRegistrationForm):
             defaultFields['password'].field.required = False
             defaultFields['password_ctl'].field.required = False
             settings = self._get_security_settings()
-            if not settings.enable_user_pwd_choice:
-                defaultFields['mail_me'].field.default = True
-            else:
-                defaultFields['mail_me'].field.default = False
+            defaultFields['mail_me'].field.default =\
+                not settings.enable_user_pwd_choice
 
         # Append the manager-focused fields
-        portal_props = getToolByName(self.context, 'portal_properties')
-        props = portal_props.site_properties
-        many_groups = props.getProperty('many_groups', False)
+        user_group_settings = getAdapter(
+            aq_inner(self.context),
+            IUserGroupsSettingsSchema,
+        )
+        many_groups = user_group_settings.many_groups
         if not many_groups:
             allFields = defaultFields + field.Fields(IAddUserSchema)
             allFields['groups'].widgetFactory = CheckBoxFieldWidget
@@ -700,21 +693,25 @@ class AddUserForm(BaseRegistrationForm):
 
         self.handle_join_success(data)
 
+        if not self._finishedRegister:
+            return
+
         portal_groups = getToolByName(self.context, 'portal_groups')
         user_id = data['user_id']
         is_zope_manager = getSecurityManager().checkPermission(
-            ManagePortal, self.context)
-
+            ManagePortal,
+            self.context,
+        )
         try:
             # Add user to the selected group(s)
-            if 'groups' in data.keys():
+            if data.get('groups', None) is not None:
                 for groupname in data['groups']:
                     group = portal_groups.getGroupById(groupname)
                     if 'Manager' in group.getRoles() and not is_zope_manager:
                         raise Forbidden
                     portal_groups.addPrincipalToGroup(user_id, groupname,
                                                       self.request)
-        except (AttributeError, ValueError), err:
+        except (AttributeError, ValueError) as err:
             IStatusMessage(self.request).addStatusMessage(err, type="error")
             return
 
@@ -724,4 +721,3 @@ class AddUserForm(BaseRegistrationForm):
             self.context.absolute_url() +
             '/@@usergroup-userprefs?searchstring=' + user_id)
 
-        self._finishedRegister = True
